﻿/*
 * Written by Stewart Berman 2012
 * Please send any bug reports / suggestions to sab@saberman.com
 * 
 */

namespace Wintellect.Interop.Win32API.SQL
{

    using System;
    using System.Runtime.InteropServices;
    using System.ComponentModel;
    using System.Text;

    public enum SQLReturnCodes : short
    {
        SQL_ERROR = -1,
        SQL_INVALID_HANDLE = -2,
        SQL_NO_DATA_FOUND = 100,
        SQL_SUCCESS = 0,
        SQL_SUCCESS_WITH_INFO = 1
    }

    public enum SQLErrors : int
    {
        SQL_NULL_HENV = 0,
        SQL_NULL_HDBC  = 0,
        SQL_NULL_HSTMT = 0
    }

    public enum SQLUsefull : short
    {
        SQL_NTS = -3,
        SQL_DROP = 1,
        SQL_SQLSTATE_SIZE = 5,
        SQL_MAX_MESSAGE_LENGTH = 512,
        SQL_MAX_DSN_LENGTH = 32
    }

    public enum SQLDrvCnnct : int
    {
        SQL_DRIVER_NOPROMPT = 0,
        SQL_DRIVER_COMPLETE = 1,
        SQL_DRIVER_PROMPT = 2,
        SQL_DRIVER_COMPLETE_REQUIRED = 3,
        SQL_DRIVER_HSTMT = 5
    }

    public enum SQLFetchType : short
    {
        SQL_FETCH_NEXT = 1,
        SQL_FETCH_FIRST = 2,
        SQL_FETCH_LAST = 3,
        SQL_FETCH_PRIOR = 4,
        SQL_FETCH_ABSOLUTE = 5,
        SQL_FETCH_RELATIVE = 6,
        SQL_FETCH_BOOKMARK = 8
    }

    sealed class Win32APISQL
    {
        [DllImport("odbc32.dll", SetLastError = true)]
        public static extern SQLReturnCodes SQLAllocConnect(IntPtr henv, out IntPtr phdbc);

        [DllImport("odbc32.dll", SetLastError = true)]
        public static extern SQLReturnCodes SQLAllocEnv(out IntPtr EnvironmentHandle);

        [DllImport("odbc32.dll", SetLastError = true)]
        public static extern SQLReturnCodes SQLAllocStmt(IntPtr henv, out IntPtr hstmt);

        [DllImport("odbc32.dll", SetLastError = true)]
        public static extern SQLReturnCodes SQLConnect(IntPtr hdbc, string sDSN, SQLUsefull DSNtype, String sUserid, SQLUsefull UseridType, String sPassword, SQLUsefull PasswordType);

        [DllImport("odbc32.dll", SetLastError = true)]
        public static extern SQLReturnCodes SQLDataSources(IntPtr henv, short fDirection, StringBuilder szDSN, short cbDSNMax, out short cbDSN, StringBuilder szDescription, short cbDescriptionMax, out short cbDescription);

        [DllImport("odbc32.dll", SetLastError = true)]
        public static extern SQLReturnCodes SQLDriverConnect(IntPtr hdbc, IntPtr hwnd, String szCSIn, short cbcsin, StringBuilder szcsout, short cbcsmax, out short cbcsout, short fDrvrcomp);

        [DllImport("odbc32.dll", SetLastError = true)]
        public static extern SQLReturnCodes SQLDrivers(IntPtr henv, short fDirection, StringBuilder szDriverDesc, short cbDriverDescMax, out short cbDriverDescription, StringBuilder szDriverAttr, short cbDriverAttrMax, out short cbDrvRaTTR);

        [DllImport("odbc32.dll", SetLastError = true)]
        public static extern SQLReturnCodes SQLDisconnect(IntPtr hdbc);

        [DllImport("odbc32.dll", SetLastError = true)]
        public static extern SQLReturnCodes SQLError(IntPtr henv, IntPtr hdbc, IntPtr hstmt, StringBuilder szQqlState, out UInt32 fNativeError, StringBuilder szErrorMsg, UInt32 cbErrorMsgMax, out UInt32 cbErrorMsg);

        [DllImport("odbc32.dll", SetLastError = true)]
        public static extern SQLReturnCodes SQLFreeConnect(IntPtr hdbc);

        [DllImport("odbc32.dll", SetLastError = true)]
        public static extern SQLReturnCodes SQLFreeEnv(IntPtr henv);

        [DllImport("odbc32.dll", SetLastError = true)]
        public static extern SQLReturnCodes SQLFreeStmt(IntPtr hstmt, int fOptions);

        [DllImport("odbc32.dll", SetLastError = true)]
        public static extern SQLReturnCodes SQLGetFunctions(IntPtr hdbc, short fDirection, out UInt16[] pfExists);

        [DllImport("odbc32.dll", SetLastError = true)]
        public static extern SQLReturnCodes SQLGetFunctions(IntPtr hdbc, short fDirection, out UInt16 pfExists);

        [DllImport("odbc32.dll", SetLastError = true)]
        public static extern SQLReturnCodes SQLGetInfo(IntPtr hdbc, short fDirection, out UInt32 lBuffer, UInt32 nBufferLen, out UInt32 nBufferUsed);

        [DllImport("odbc32.dll", SetLastError = true)]
        public static extern SQLReturnCodes SQLGetInfo(IntPtr hdbc, short fDirection, StringBuilder sBuffer, UInt32 nBufferLen, out UInt32 nBufferUsed);

        [DllImport("user32.dll", CharSet = CharSet.Auto, ExactSpelling = true)]
        public static extern IntPtr GetActiveWindow();
    }
}



/*

'
'|========================================================================|
'| ODBC Module Core Definitions                                           |
'|========================================================================|
'
'  ODBC Core API's Definitions -- 32 bit versions
'
Declare Function SQLAllocConnect Lib "odbc32.dll" (ByVal henv&, phdbc&) As Integer
Declare Function SQLAllocEnv Lib "odbc32.dll" (phenv&) As Integer
Declare Function SQLAllocStmt Lib "odbc32.dll" (ByVal hdbc&, phstmt&) As Integer
Declare Function SQLBindCol Lib "odbc32.dll" (ByVal hstmt&, ByVal icol%, ByVal fCType%, rgbValue As Any, ByVal cbValueMax&, pcbValue&) As Integer
Declare Function SQLCancel Lib "odbc32.dll" (ByVal hstmt&) As Integer

Declare Function SQLColAttributes Lib "odbc32.dll" (ByVal hstmt&, ByVal icol%, ByVal fDescType%, rgbDesc As Any, ByVal cbDescMax%, pcbDesc%, pfDesc&) As Integer
Declare Function SQLColAttributesString Lib "odbc32.dll" Alias "SQLColAttributes" (ByVal hstmt&, ByVal icol%, ByVal fDescType%, ByVal rgbDesc As String, ByVal cbDescMax%, pcbDesc%, pfDesc&) As Integer

Declare Function SQLConnect Lib "odbc32.dll" (ByVal hdbc&, ByVal szDSN$, ByVal cbDSN%, ByVal szUID$, ByVal cbUID%, ByVal szAuthStr$, ByVal cbAuthStr%) As Integer
Declare Function SQLDescribeCol Lib "odbc32.dll" (ByVal hstmt&, ByVal icol%, ByVal szColName$, ByVal cbColNameMax%, pcbColName%, pfSqlType%, pcbColDef&, pibScale%, pfNullable%) As Integer
Declare Function SQLDisconnect Lib "odbc32.dll" (ByVal hdbc&) As Integer
Declare Function SQLError Lib "odbc32.dll" (ByVal henv&, ByVal hdbc&, ByVal hstmt&, ByVal szSqlState$, pfNativeError&, ByVal szErrorMsg$, ByVal cbErrorMsgMax%, pcbErrorMsg%) As Integer
Declare Function SQLExecDirect Lib "odbc32.dll" (ByVal hstmt&, ByVal szSqlStr$, ByVal cbSqlStr&) As Integer
Declare Function SQLExecute Lib "odbc32.dll" (ByVal hstmt&) As Integer
Declare Function SQLFetch Lib "odbc32.dll" (ByVal hstmt&) As Integer
Declare Function SQLFreeConnect Lib "odbc32.dll" (ByVal hdbc&) As Integer
Declare Function SQLFreeEnv Lib "odbc32.dll" (ByVal henv&) As Integer
Declare Function SQLFreeStmt Lib "odbc32.dll" (ByVal hstmt&, ByVal fOption%) As Integer
Declare Function SQLGetCursorName Lib "odbc32.dll" (ByVal hstmt&, ByVal szCursor$, ByVal cbCursorMax%, pcbCursor%) As Integer
Declare Function SQLNumResultCols Lib "odbc32.dll" (ByVal hstmt&, pccol%) As Integer
Declare Function SQLPrepare Lib "odbc32.dll" (ByVal hstmt&, ByVal szSqlStr$, ByVal cbSqlStr&) As Integer
Declare Function SQLRowCount Lib "odbc32.dll" (ByVal hstmt&, pcrow&) As Integer
Declare Function SQLSetCursorName Lib "odbc32.dll" (ByVal hstmt&, ByVal szCursor$, ByVal cbCursor%) As Integer
Declare Function SQLSetParam Lib "odbc32.dll" (ByVal hstmt&, ByVal ipar%, ByVal fCType%, ByVal fSqlType%, ByVal cbColDef&, ByVal ibScale%, rgbValue As Any, pcbValue&) As Integer
Declare Function SQLTransact Lib "odbc32.dll" (ByVal henv&, ByVal hdbc&, ByVal fType%) As Integer
'
'|========================================================================|
'| ODBC Module Extended Definitions                                       |
'|========================================================================|
''  Level 1 Prototypes
'
Declare Function SQLBindParameter Lib "odbc32.dll" (ByVal hstmt&, ByVal ipar%, ByVal fParamType%, ByVal fCType%, ByVal fSqlType%, ByVal cbColDef&, ByVal ibScale%, rgbValue As Any, ByVal cbValueMax&, pcbValue As Long) As Integer
Declare Function SQLColumns Lib "odbc32.dll" (ByVal hstmt&, szTblQualifier As Any, ByVal cbTblQualifier%, szTblOwner As Any, ByVal cbTblOwner%, szTblName As Any, ByVal cbTblName%, szColName As Any, ByVal cbColName%) As Integer
Declare Function SQLDriverConnect Lib "odbc32.dll" (ByVal hdbc&, ByVal hWnd As Long, ByVal szCSIn$, ByVal cbCSIn%, ByVal szCSOut$, ByVal cbCSMax%, cbCSOut%, ByVal fDrvrComp%) As Integer

Declare Function SQLGetConnectOption Lib "odbc32.dll" (ByVal hdbc&, ByVal fOption%, ByRef pvParam As Any) As Integer
Declare Function SQLGetConnectOptionString Lib "odbc32.dll" Alias "SQLGetConnectOption" (ByVal hdbc&, ByVal fOption%, ByVal pvParam As String) As Integer

'Declare Function SQLGetData Lib "odbc32.dll" (ByVal hstmt&, ByVal icol%, ByVal fCType%, ByVal rgbValue As String, ByVal cbValueMax&, pcbValue&) As Integer
Declare Function SQLGetData Lib "odbc32.dll" (ByVal hstmt&, ByVal icol%, ByVal fCType%, rgbValue As Any, ByVal cbValueMax&, pcbValue&) As Integer
Declare Function SQLGetNumericData Lib "odbc32.dll" Alias "SQLGetData" (ByVal hstmt&, ByVal icol%, ByVal fCType%, ByRef rgbValue As Any, ByVal cbValueMax&, pcbValue&) As Integer

Declare Function SQLGetFunctions Lib "odbc32.dll" (ByVal hdbc&, ByVal fFunction%, pfExists%) As Integer

Declare Function SQLGetInfo Lib "odbc32.dll" (ByVal hdbc&, ByVal fInfoType%, ByRef rgbInfoValue As Any, ByVal cbInfoMax%, cbInfoOut%) As Integer
Declare Function SQLGetInfoString Lib "odbc32.dll" Alias "SQLGetInfo" (ByVal hdbc&, ByVal fInfoType%, ByVal rgbInfoValue As String, ByVal cbInfoMax%, cbInfoOut%) As Integer

Declare Function SQLGetStmtOption Lib "odbc32.dll" (ByVal hstmt&, ByVal fOption%, ByRef pvParam As Any) As Integer
Declare Function SQLGetStmtOptionString Lib "odbc32.dll" Alias "SQLGetStmtOption" (ByVal hstmt&, ByVal fOption%, ByVal pvParam As String) As Integer

Declare Function SQLGetTypeInfo Lib "odbc32.dll" (ByVal hstmt&, ByVal fSqlType%) As Integer
Declare Function SQLParamData Lib "odbc32.dll" (ByVal hstmt&, prgbValue As Any) As Integer
Declare Function SQLPutData Lib "odbc32.dll" (ByVal hstmt&, rgbValue As Any, ByVal cbValue&) As Integer
Declare Function SQLSetConnectOption Lib "odbc32.dll" (ByVal hdbc&, ByVal fOption%, ByVal vParam As Any) As Integer
Declare Function SQLSetConnectStringOption Lib "odbc32.dll" Alias "SQLSetConnectOption" (ByVal hdbc&, ByVal fOption%, vParam$) As Integer

Declare Function SQLSetStmtOption Lib "odbc32.dll" (ByVal hstmt&, ByVal fOption%, ByVal vParam&) As Integer
Declare Function SQLSpecialColumns Lib "odbc32.dll" (ByVal hstmt&, ByVal fColType%, szTblQualifier As Any, ByVal cbTblQualifier%, szTblOwner As Any, ByVal cbTblOwner%, szTblName As Any, ByVal cbTblName%, ByVal fScope%, ByVal fNullable%) As Integer
Declare Function SQLStatistics Lib "odbc32.dll" (ByVal hstmt&, szTblQualifier As Any, ByVal cbTblQualifier%, szTblOwner As Any, ByVal cbTblOwner%, szTblName As Any, ByVal cbTblName%, ByVal fUnique%, ByVal fAccuracy%) As Integer
Declare Function SQLTables Lib "odbc32.dll" (ByVal hstmt&, szTblQualifier As Any, ByVal cbTblQualifier%, szTblOwner As Any, ByVal cbTblOwner%, szTblName As Any, ByVal cbTblName%, szTblType As Any, ByVal cbTblType%) As Integer

'  Level 2 Prototypes
'
Declare Function SQLBrowseConnect Lib "odbc32.dll" (ByVal hdbc&, ByVal szConnStrIn$, ByVal cbConnStrIn%, ByVal szConnStrOut$, ByVal cbConnStrOutMax%, pcbConnStrOut%) As Integer
Declare Function SQLColumnPrivileges Lib "odbc32.dll" (ByVal hstmt&, szTQf As Any, ByVal cbTQf%, szTOwn As Any, ByVal cbTOwn%, szTName As Any, ByVal cbTName%, szColName As Any, ByVal cbColName%) As Integer
Declare Function SQLDrivers Lib "odbc32.dll" (ByVal henv&, ByVal fDirection%, ByVal szDriverDesc$, ByVal cbDriverDescMax%, pcbDriverDesc%, ByVal szDriverAttr$, ByVal cbDrvrAttrMax%, pcbDrvrAttr%) As Integer
Declare Function SQLDataSources Lib "odbc32.dll" (ByVal henv&, ByVal fDirection%, ByVal szDSN$, ByVal cbDSNMax%, pcbDSN%, ByVal szDescription$, ByVal cbDescriptionMax%, pcbDescription%) As Integer
Declare Function SQLDescribeParam Lib "odbc32.dll" (ByVal hstmt&, ByVal ipar%, pfSqlType%, pcbColDef&, pibScale%, pfNullable%) As Integer
Declare Function SQLExtendedFetch Lib "odbc32.dll" (ByVal hstmt&, ByVal fFetchType%, ByVal irow&, pcrow&, rgfRowStatus%) As Integer
Declare Function SQLForeignKeys Lib "odbc32.dll" (ByVal hstmt&, ByVal PTQual&, ByVal PTQual%, ByVal PTOwnr&, ByVal PTOwnr%, ByVal PTName&, ByVal PTName%, ByVal FTQual&, ByVal FTQf%, ByVal FTOwnr&, ByVal FTOwnr%, ByVal FTName&, ByVal FTName%) As Integer
Declare Function SQLMoreResults Lib "odbc32.dll" (ByVal hstmt&) As Integer
Declare Function SQLNativeSql Lib "odbc32.dll" (ByVal hdbc&, ByVal szSqlStrIn$, ByVal cbSqlStrIn&, ByVal szSqlStr$, ByVal cbSqlStrMax&, pcbSqlStr&) As Integer
Declare Function SQLNumParams Lib "odbc32.dll" (ByVal hstmt&, pcpar%) As Integer
Declare Function SQLParamOptions Lib "odbc32.dll" (ByVal hstmt&, ByVal crow%, pirow&) As Integer
Declare Function SQLPrimaryKeys Lib "odbc32.dll" (ByVal hstmt&, szTblQualifier As Any, ByVal cbTblQualifier%, szTblOwner As Any, ByVal cbTblOwner%, szTblName As Any, ByVal cbTblName%) As Integer
Declare Function SQLProcedureColumns Lib "odbc32.dll" (ByVal hstmt&, szProcQualifier As Any, ByVal cbProcQualifier%, szProcOwner As Any, ByVal cbProcOwner%, szProcName As Any, ByVal cbProcName%, szColName As Any, ByVal cbColName%) As Integer
Declare Function SQLProcedures Lib "odbc32.dll" (ByVal hstmt&, szProcQualifier As Any, ByVal cbProcQualifier%, szProcOwner As Any, ByVal cbProcOwner%, szProcName As Any, ByVal cbProcName%) As Integer
Declare Function SQLSetPos Lib "odbc32.dll" (ByVal hstmt&, ByVal irow%, ByVal fOption%, ByVal fLock%) As Integer
Declare Function SQLSetScrollOptions Lib "odbc32.dll" (ByVal hstmt&, ByVal fConcurrency%, ByVal crowKeyset&, ByVal crowRowset%) As Integer
Declare Function SQLTablePrivileges Lib "odbc32.dll" (ByVal hstmt&, szTblQualifier As Any, ByVal cbTblQualifier%, szTblOwner As Any, ByVal cbTblOwner%, szTblName As Any, ByVal cbTblName%) As Integer
*/



public enum ODBCconst : int
{
//
//  32 Bit declares
//  ODBC Constants/Types
//
//  generally useful constants
//
    SQL_NTS = -3,                  //  NTS = Null Terminated String
    SQL_SQLSTATE_SIZE = 5,         //  size of SQLSTATE
    SQL_MAX_MESSAGE_LENGTH = 512,  //  message buffer size
    SQL_MAX_DSN_LENGTH = 32,       //  maximum data source name size

//  RETCODEs
//
    SQL_ERROR = -1,
    SQL_INVALID_HANDLE = -2,
    SQL_NO_DATA_FOUND = 100,
    SQL_SUCCESS = 0,
    SQL_SUCCESS_WITH_INFO = 1,

//  SQLFreeStmt defines
//
    SQL_CLOSE = 0,
    SQL_DROP = 1,
    SQL_UNBIND = 2,
    SQL_RESET_PARAMS = 3,

//  SQLSetParam defines
//
    SQL_C_DEFAULT = 99,

//  SQLTransact defines
//
    SQL_COMMIT = 0,
    SQL_ROLLBACK = 1,

//  Standard SQL datatypes, using ANSI type numbering
//
    SQL_CHAR = 1,
    SQL_NUMERIC = 2,
    SQL_DECIMAL = 3,
    SQL_INTEGER = 4,
    SQL_SMALLINT = 5,
    SQL_FLOAT = 6,
    SQL_REAL = 7,
    SQL_DOUBLE = 8,
    SQL_VARCHAR = 12,

    SQL_TYPE_MIN = 1,
    SQL_TYPE_NULL = 0,
    SQL_TYPE_MAX = 12,

//  C datatype to SQL datatype mapping    SQL types
//
    SQL_C_CHAR = SQL_CHAR,        //  CHAR, VARCHAR, DECIMAL, NUMERIC
    SQL_C_LONG = SQL_INTEGER,      //  INTEGER
    SQL_C_SHORT = SQL_SMALLINT,    //  SMALLINT
    SQL_C_FLOAT = SQL_REAL,        //  REAL
    SQL_C_DOUBLE = SQL_DOUBLE,     //  FLOAT, DOUBLE

//  NULL status constants.  These are used in SQLColumns, SQLColAttributes,
//  SQLDescribeCol, and SQLSpecialColumns to describe the nullablity of a
//  column in a table.  SQL_NULLABLE_UNKNOWN can be returned only by
//  SQLDescribeCol or SQLColAttributes.  It is used when the DBMS//s meta-data
//  does not contain this info.
//
    SQL_NO_NULLS = 0,
    SQL_NULLABLE = 1,
    SQL_NULLABLE_UNKNOWN = 2,

//  Special length values
//
    SQL_NULL_DATA = -1,
    SQL_DATA_AT_EXEC = -2,

//  SQLColAttributes defines
//
    SQL_COLUMN_COUNT = 0,
    SQL_COLUMN_NAME = 1,
    SQL_COLUMN_TYPE = 2,
    SQL_COLUMN_LENGTH = 3,
    SQL_COLUMN_PRECISION = 4,
    SQL_COLUMN_SCALE = 5,
    SQL_COLUMN_DISPLAY_SIZE = 6,
    SQL_COLUMN_NULLABLE = 7,
    SQL_COLUMN_UNSIGNED = 8,
    SQL_COLUMN_MONEY = 9,
    SQL_COLUMN_UPDATABLE = 10,
    SQL_COLUMN_AUTO_INCREMENT = 11,
    SQL_COLUMN_CASE_SENSITIVE = 12,
    SQL_COLUMN_SEARCHABLE = 13,
    SQL_COLUMN_TYPE_NAME = 14,
    SQL_COLUMN_TABLE_NAME = 15,
    SQL_COLUMN_OWNER_NAME = 16,
    SQL_COLUMN_QUALIFIER_NAME = 17,
    SQL_COLUMN_LABEL = 18,
    SQL_COLATT_OPT_MAX = SQL_COLUMN_LABEL,

//  SQLColAttributes subdefines for SQL_COLUMN_UPDATABLE
//
    SQL_ATTR_READONLY = 0,
    SQL_ATTR_WRITE = 1,
    SQL_ATTR_READWRITE_UNKNOWN = 2,

//  SQLColAttributes subdefines for SQL_COLUMN_SEARCHABLE
//  These are also used by SQLGetInfo
//
    SQL_UNSEARCHABLE = 0,
    SQL_LIKE_ONLY = 1,
    SQL_ALL_EXCEPT_LIKE = 2,
    SQL_SEARCHABLE = 3,

//  SQLError defines
//
    SQL_NULL_HENV = 0,
    SQL_NULL_HDBC = 0,
    SQL_NULL_HSTMT = 0,


//|========================================================================|
//| ODBC Global Extended Definitions                                       |
//|========================================================================|

// Level 1 Definitions/Functions
// Generally useful constants
//
    SQL_MAX_OPTION_STRING_LENGTH = 256,

// Additional return codes
//
    SQL_STILL_EXECUTING = 2,
    SQL_NEED_DATA = 99,

// SQL extended datatypes
//
    SQL_DATE = 9,
    SQL_TIME = 10,
    SQL_TIMESTAMP = 11,
    SQL_LONGVARCHAR = -1,
    SQL_BINARY = -2,
    SQL_VARBINARY = -3,
    SQL_LONGVARBINARY = -4,
    SQL_BIGINT = -5,
    SQL_TINYINT = -6,
    SQL_BIT = -7,
    SQL_TYPE_DRIVER_START = -80,

// C datatype to SQL datatype mapping
//
//
    SQL_SIGNED_OFFSET = -20,
    SQL_UNSIGNED_OFFSET = -22,

    SQL_C_DATE = SQL_DATE,
    SQL_C_TIME = SQL_TIME,
    SQL_C_TIMESTAMP = SQL_TIMESTAMP,
    SQL_C_BINARY = SQL_BINARY,
    SQL_C_BIT = SQL_BIT,
    SQL_C_TINYINT = SQL_TINYINT,
    SQL_C_SLONG = SQL_C_LONG + SQL_SIGNED_OFFSET,
    SQL_C_SSHORT = SQL_C_SHORT + SQL_SIGNED_OFFSET,
    SQL_C_STINYINT = SQL_TINYINT + SQL_SIGNED_OFFSET,
    SQL_C_ULONG = SQL_C_LONG + SQL_UNSIGNED_OFFSET,
    SQL_C_USHORT = SQL_C_SHORT + SQL_UNSIGNED_OFFSET,
    SQL_C_UTINYINT = SQL_TINYINT + SQL_UNSIGNED_OFFSET,
    SQL_C_BOOKMARK = SQL_C_ULONG,


    SQL_ALL_TYPES = 0,

// Options for SQLDriverConnect
//
    SQL_DRIVER_NOPROMPT = 0,
    SQL_DRIVER_COMPLETE = 1,
    SQL_DRIVER_PROMPT = 2,
    SQL_DRIVER_COMPLETE_REQUIRED = 3,

// Special return values for SQLGetData
//
    SQL_NO_TOTAL = -4,

// SQLSetParam extensions
//
    SQL_DEFAULT_PARAM = -5,
    SQL_IGNORE = -6,
    SQL_LEN_DATA_AT_EXEC_OFFSET = -100,


// Defines for SQLGetFunctions
// Core Functions
//
    SQL_API_SQLALLOCCONNECT = 1,
    SQL_API_SQLALLOCENV = 2,
    SQL_API_SQLALLOCSTMT = 3,
    SQL_API_SQLBINDCOL = 4,
    SQL_API_SQLCANCEL = 5,
    SQL_API_SQLCOLATTRIBUTES = 6,
    SQL_API_SQLCONNECT = 7,
    SQL_API_SQLDESCRIBECOL = 8,
    SQL_API_SQLDISCONNECT = 9,
    SQL_API_SQLERROR = 10,
    SQL_API_SQLEXECDIRECT = 11,
    SQL_API_SQLEXECUTE = 12,
    SQL_API_SQLFETCH = 13,
    SQL_API_SQLFREECONNECT = 14,
    SQL_API_SQLFREEENV = 15,
    SQL_API_SQLFREESTMT = 16,
    SQL_API_SQLGETCURSORNAME = 17,
    SQL_API_SQLNUMRESULTCOLS = 18,
    SQL_API_SQLPREPARE = 19,
    SQL_API_SQLROWCOUNT = 20,
    SQL_API_SQLSETCURSORNAME = 21,
    SQL_API_SQLSETPARAM = 22,
    SQL_API_SQLTRANSACT = 23,
    SQL_NUM_FUNCTIONS = 23,
    SQL_EXT_API_START = 40,

// Level 1 Functions
//
    SQL_API_SQLCOLUMNS = 40,
    SQL_API_SQLDRIVERCONNECT = 41,
    SQL_API_SQLGETCONNECTOPTION = 42,
    SQL_API_SQLGETDATA = 43,
    SQL_API_SQLGETFUNCTIONS = 44,
    SQL_API_SQLGETINFO = 45,
    SQL_API_SQLGETSTMTOPTION = 46,
    SQL_API_SQLGETTYPEINFO = 47,
    SQL_API_SQLPARAMDATA = 48,
    SQL_API_SQLPUTDATA = 49,
    SQL_API_SQLSETCONNECTOPTION = 50,
    SQL_API_SQLSETSTMTOPTION = 51,
    SQL_API_SQLSPECIALCOLUMNS = 52,
    SQL_API_SQLSTATISTICS = 53,
    SQL_API_SQLTABLES = 54,

// Level 2 Functions
//
    SQL_API_SQLBROWSECONNECT = 55,
    SQL_API_SQLCOLUMNPRIVILEGES = 56,
    SQL_API_SQLDATASOURCES = 57,
    SQL_API_SQLDESCRIBEPARAM = 58,
    SQL_API_SQLEXTENDEDFETCH = 59,
    SQL_API_SQLFOREIGNKEYS = 60,
    SQL_API_SQLMORERESULTS = 61,
    SQL_API_SQLNATIVESQL = 62,
    SQL_API_SQLNUMPARAMS = 63,
    SQL_API_SQLPARAMOPTIONS = 64,
    SQL_API_SQLPRIMARYKEYS = 65,
    SQL_API_SQLPROCEDURECOLUMNS = 66,
    SQL_API_SQLPROCEDURES = 67,
    SQL_API_SQLSETPOS = 68,
    SQL_API_SQLSETSCROLLOPTIONS = 69,
    SQL_API_SQLTABLEPRIVILEGES = 70,
    SQL_API_SQLDRIVERS = 71,
    SQL_API_SQLBINDPARAMETER = 72,

    SQL_EXT_API_LAST = 72,
    SQL_API_ALL_FUNCTIONS = 0,

    SQL_NUM_EXTENSIONS = (SQL_EXT_API_LAST - SQL_EXT_API_START + 1),

// Defines for SQLGetInfo
//
    SQL_INFO_FIRST = 0,
    SQL_ACTIVE_CONNECTIONS = 0,
    SQL_ACTIVE_STATEMENTS = 1,
    SQL_DATA_SOURCE_NAME = 2,
    SQL_DRIVER_HDBC = 3,
    SQL_DRIVER_HENV = 4,
    SQL_DRIVER_HSTMT = 5,
    SQL_DRIVER_NAME = 6,
    SQL_DRIVER_VER = 7,
    SQL_FETCH_DIRECTION = 8,
    SQL_ODBC_API_CONFORMANCE = 9,
    SQL_ODBC_VER = 10,
    SQL_ROW_UPDATES = 11,
    SQL_ODBC_SAG_CLI_CONFORMANCE = 12,
    SQL_SERVER_NAME = 13,
    SQL_SEARCH_PATTERN_ESCAPE = 14,
    SQL_ODBC_SQL_CONFORMANCE = 15,

    SQL_DBMS_NAME = 17,
    SQL_DBMS_VER = 18,

    SQL_ACCESSIBLE_TABLES = 19,
    SQL_ACCESSIBLE_PROCEDURES = 20,
    SQL_PROCEDURES = 21,
    SQL_CONCAT_NULL_BEHAVIOR = 22,
    SQL_CURSOR_COMMIT_BEHAVIOR = 23,
    SQL_CURSOR_ROLLBACK_BEHAVIOR = 24,
    SQL_DATA_SOURCE_READ_ONLY = 25,
    SQL_DEFAULT_TXN_ISOLATION = 26,
    SQL_EXPRESSIONS_IN_ORDERBY = 27,
    SQL_IDENTIFIER_CASE = 28,
    SQL_IDENTIFIER_QUOTE_CHAR = 29,
    SQL_MAX_COLUMN_NAME_LEN = 30,
    SQL_MAX_CURSOR_NAME_LEN = 31,
    SQL_MAX_OWNER_NAME_LEN = 32,
    SQL_MAX_PROCEDURE_NAME_LEN = 33,
    SQL_MAX_QUALIFIER_NAME_LEN = 34,
    SQL_MAX_TABLE_NAME_LEN = 35,
    SQL_MULT_RESULT_SETS = 36,
    SQL_MULTIPLE_ACTIVE_TXN = 37,
    SQL_OUTER_JOINS = 38,
    SQL_OWNER_TERM = 39,
    SQL_PROCEDURE_TERM = 40,
    SQL_QUALIFIER_NAME_SEPARATOR = 41,
    SQL_QUALIFIER_TERM = 42,
    SQL_SCROLL_CONCURRENCY = 43,
    SQL_SCROLL_OPTIONS = 44,
    SQL_TABLE_TERM = 45,
    SQL_TXN_CAPABLE = 46,
    SQL_USER_NAME = 47,

    SQL_CONVERT_FUNCTIONS = 48,
    SQL_NUMERIC_FUNCTIONS = 49,
    SQL_STRING_FUNCTIONS = 50,
    SQL_SYSTEM_FUNCTIONS = 51,
    SQL_TIMEDATE_FUNCTIONS = 52,

    SQL_CONVERT_BIGINT = 53,
    SQL_CONVERT_BINARY = 54,
    SQL_CONVERT_BIT = 55,
    SQL_CONVERT_CHAR = 56,
    SQL_CONVERT_DATE = 57,
    SQL_CONVERT_DECIMAL = 58,
    SQL_CONVERT_DOUBLE = 59,
    SQL_CONVERT_FLOAT = 60,
    SQL_CONVERT_INTEGER = 61,
    SQL_CONVERT_LONGVARCHAR = 62,
    SQL_CONVERT_NUMERIC = 63,
    SQL_CONVERT_REAL = 64,
    SQL_CONVERT_SMALLINT = 65,
    SQL_CONVERT_TIME = 66,
    SQL_CONVERT_TIMESTAMP = 67,
    SQL_CONVERT_TINYINT = 68,
    SQL_CONVERT_VARBINARY = 69,
    SQL_CONVERT_VARCHAR = 70,
    SQL_CONVERT_LONGVARBINARY = 71,

    SQL_TXN_ISOLATION_OPTION = 72,
    SQL_ODBC_SQL_OPT_IEF = 73,
    SQL_CORRELATION_NAME = 74,
    SQL_NON_NULLABLE_COLUMNS = 75,
    SQL_DRIVER_HLIB = 76,
    SQL_DRIVER_ODBC_VER = 77,
    SQL_LOCK_TYPES = 78,
    SQL_POS_OPERATIONS = 79,
    SQL_POSITIONED_STATEMENTS = 80,
    SQL_GETDATA_EXTENSIONS = 81,
    SQL_BOOKMARK_PERSISTENCE = 82,
    SQL_STATIC_SENSITIVITY = 83,
    SQL_FILE_USAGE = 84,
    SQL_NULL_COLLATION = 85,
    SQL_ALTER_TABLE = 86,
    SQL_COLUMN_ALIAS = 87,
    SQL_GROUP_BY = 88,
    SQL_KEYWORDS = 89,
    SQL_ORDER_BY_COLUMNS_IN_SELECT = 90,
    SQL_OWNER_USAGE = 91,
    SQL_QUALIFIER_USAGE = 92,
    SQL_QUOTED_IDENTIFIER_CASE = 93,
    SQL_SPECIAL_CHARACTERS = 94,
    SQL_SUBQUERIES = 95,
    SQL_UNION = 96,
    SQL_MAX_COLUMNS_IN_GROUP_BY = 97,
    SQL_MAX_COLUMNS_IN_INDEX = 98,
    SQL_MAX_COLUMNS_IN_ORDER_BY = 99,
    SQL_MAX_COLUMNS_IN_SELECT = 100,
    SQL_MAX_COLUMNS_IN_TABLE = 101,
    SQL_MAX_INDEX_SIZE = 102,
    SQL_MAX_ROW_SIZE_INCLUDES_LONG = 103,
    SQL_MAX_ROW_SIZE = 104,
    SQL_MAX_STATEMENT_LEN = 105,
    SQL_MAX_TABLES_IN_SELECT = 106,
    SQL_MAX_USER_NAME_LEN = 107,
    SQL_MAX_CHAR_LITERAL_LEN = 108,
    SQL_TIMEDATE_ADD_INTERVALS = 109,
    SQL_TIMEDATE_DIFF_INTERVALS = 110,
    SQL_NEED_LONG_DATA_LEN = 111,
    SQL_MAX_BINARY_LITERAL_LEN = 112,
    SQL_LIKE_ESCAPE_CLAUSE = 113,
    SQL_QUALIFIER_LOCATION = 114,


    SQL_INFO_LAST = SQL_QUALIFIER_LOCATION,
    SQL_INFO_DRIVER_START = 1000,

// "SQL_CONVERT_" return value bitmasks,
//
    SQL_CVT_CHAR = 0x1,
    SQL_CVT_NUMERIC = 0x2,
    SQL_CVT_DECIMAL = 0x4,
    SQL_CVT_INTEGER = 0x8,
    SQL_CVT_SMALLINT = 0x10,
    SQL_CVT_FLOAT = 0x20,
    SQL_CVT_REAL = 0x40,
    SQL_CVT_DOUBLE = 0x80,
    SQL_CVT_VARCHAR = 0x100,
    SQL_CVT_LONGVARCHAR = 0x200,
    SQL_CVT_BINARY = 0x400,
    SQL_CVT_VARBINARY = 0x800,
    SQL_CVT_BIT = 0x1000,
    SQL_CVT_TINYINT = 0x2000,
    SQL_CVT_BIGINT = 0x4000,
    SQL_CVT_DATE = 0x8000,
    SQL_CVT_TIME = 0x10000,
    SQL_CVT_TIMESTAMP = 0x20000,
    SQL_CVT_LONGVARBINARY = 0x40000,


// Conversion functions
//
    SQL_FN_CVT_CONVERT = 0x1,

// String functions
//
    SQL_FN_STR_CONCAT = 0x1,
    SQL_FN_STR_INSERT = 0x2,
    SQL_FN_STR_LEFT = 0x4,
    SQL_FN_STR_LTRIM = 0x8,
    SQL_FN_STR_LENGTH = 0x10,
    SQL_FN_STR_LOCATE = 0x20,
    SQL_FN_STR_LCASE = 0x40,
    SQL_FN_STR_REPEAT = 0x80,
    SQL_FN_STR_REPLACE = 0x100,
    SQL_FN_STR_RIGHT = 0x200,
    SQL_FN_STR_RTRIM = 0x400,
    SQL_FN_STR_SUBSTRING = 0x800,
    SQL_FN_STR_UCASE = 0x1000,
    SQL_FN_STR_ASCII = 0x2000,
    SQL_FN_STR_CHAR = 0x4000,
    SQL_FN_STR_DIFFERENCE = 0x8000,
    SQL_FN_STR_LOCATE_2 = 0x10000,
    SQL_FN_STR_SOUNDEX = 0x20000,
    SQL_FN_STR_SPACE = 0x40000,


// Numeric functions
//
    SQL_FN_NUM_ABS = 0x1,
    SQL_FN_NUM_ACOS = 0x2,
    SQL_FN_NUM_ASIN = 0x4,
    SQL_FN_NUM_ATAN = 0x8,
    SQL_FN_NUM_ATAN2 = 0x10,
    SQL_FN_NUM_CEILING = 0x20,
    SQL_FN_NUM_COS = 0x40,
    SQL_FN_NUM_COT = 0x80,
    SQL_FN_NUM_EXP = 0x100,
    SQL_FN_NUM_FLOOR = 0x200,
    SQL_FN_NUM_LOG = 0x400,
    SQL_FN_NUM_MOD = 0x800,
    SQL_FN_NUM_SIGN = 0x1000,
    SQL_FN_NUM_SIN = 0x2000,
    SQL_FN_NUM_SQRT = 0x4000,
    SQL_FN_NUM_TAN = 0x8000,
    SQL_FN_NUM_PI = 0x10000,
    SQL_FN_NUM_RAND = 0x20000,
    SQL_FN_NUM_DEGREES = 0x40000,
    SQL_FN_NUM_LOG10 = 0x80000,
    SQL_FN_NUM_POWER = 0x100000,
    SQL_FN_NUM_RADIANS = 0x200000,
    SQL_FN_NUM_ROUND = 0x400000,
    SQL_FN_NUM_TRUNCATE = 0x800000,


// Time/date functions
//
    SQL_FN_TD_NOW = 0x1,
    SQL_FN_TD_CURDATE = 0x2,
    SQL_FN_TD_DAYOFMONTH = 0x4,
    SQL_FN_TD_DAYOFWEEK = 0x8,
    SQL_FN_TD_DAYOFYEAR = 0x10,
    SQL_FN_TD_MONTH = 0x20,
    SQL_FN_TD_QUARTER = 0x40,
    SQL_FN_TD_WEEK = 0x80,
    SQL_FN_TD_YEAR = 0x100,
    SQL_FN_TD_CURTIME = 0x200,
    SQL_FN_TD_HOUR = 0x400,
    SQL_FN_TD_MINUTE = 0x800,
    SQL_FN_TD_SECOND = 0x1000,
//
    SQL_FN_TD_TIMESTAMPADD = 0x2000,
    SQL_FN_TD_TIMESTAMPDIFF = 0x4000,
    SQL_FN_TD_DAYNAME = 0x8000,
    SQL_FN_TD_MONTHNAME = 0x10000,


// System functions
//
    SQL_FN_SYS_USERNAME = 0x1,
    SQL_FN_SYS_DBNAME = 0x2,
    SQL_FN_SYS_IFNULL = 0x4,

// Timedate intervals
//
//
    SQL_FN_TSI_FRAC_SECOND = 0x1,
    SQL_FN_TSI_SECOND = 0x2,
    SQL_FN_TSI_MINUTE = 0x4,
    SQL_FN_TSI_HOUR = 0x8,
    SQL_FN_TSI_DAY = 0x10,
    SQL_FN_TSI_WEEK = 0x20,
    SQL_FN_TSI_MONTH = 0x40,
    SQL_FN_TSI_QUARTER = 0x80,
    SQL_FN_TSI_YEAR = 0x100,


// ODBC API conformance
//
    SQL_OAC_NONE = 0,
    SQL_OAC_LEVEL1 = 1,
    SQL_OAC_LEVEL2 = 2,

// SAG CLI conformance
//
    SQL_OSCC_NOT_COMPLIANT = 0,
    SQL_OSCC_COMPLIANT = 1,

// ODBC SQL conformance
//
    SQL_OSC_MINIMUM = 0,
    SQL_OSC_CORE = 1,
    SQL_OSC_EXTENDED = 2,

// Concatenation behavior
//
    SQL_CB_NULL = 0,
    SQL_CB_NON_NULL = 1,

// Cursor commit behavior
//
    SQL_CB_DELETE = 0,
    SQL_CB_CLOSE = 1,
    SQL_CB_PRESERVE = 2,

// Identifier case
//
    SQL_IC_UPPER = 1,
    SQL_IC_LOWER = 2,
    SQL_IC_SENSITIVE = 3,
    SQL_IC_MIXED = 4,

// Transaction capable
//
    SQL_TC_NONE = 0,
    SQL_TC_DML = 1,
    SQL_TC_ALL = 2,
//
    SQL_TC_DDL_COMMIT = 3,
    SQL_TC_DDL_IGNORE = 4,


// Scroll option masks
//
    SQL_SO_FORWARD_ONLY = 0x1,
    SQL_SO_KEYSET_DRIVEN = 0x2,
    SQL_SO_DYNAMIC = 0x4,
    SQL_SO_MIXED = 0x8,
//
    SQL_SO_STATIC = 0x10,


// Scroll concurrency option masks
//
    SQL_SCCO_READ_ONLY = 0x1,
    SQL_SCCO_LOCK = 0x2,
    SQL_SCCO_OPT_ROWVER = 0x4,
    SQL_SCCO_OPT_VALUES = 0x8,

// Fetch direction option masks
//
    SQL_FD_FETCH_NEXT = 0x1,
    SQL_FD_FETCH_FIRST = 0x2,
    SQL_FD_FETCH_LAST = 0x4,
    SQL_FD_FETCH_PRIOR = 0x8,
    SQL_FD_FETCH_ABSOLUTE = 0x10,
    SQL_FD_FETCH_RELATIVE = 0x20,
    SQL_FD_FETCH_RESUME = 0x40,
//
    SQL_FD_FETCH_BOOKMARK = 0x80,


// Transaction isolation option masks
//
    SQL_TXN_READ_UNCOMMITTED = 0x1,
    SQL_TXN_READ_COMMITTED = 0x2,
    SQL_TXN_REPEATABLE_READ = 0x4,
    SQL_TXN_SERIALIZABLE = 0x8,
    SQL_TXN_VERSIONING = 0x10,

// Correlation name
//
    SQL_CN_NONE = 0,
    SQL_CN_DIFFERENT = 1,
    SQL_CN_ANY = 2,

// Non-nullable columns
//
    SQL_NNC_NULL = 0,
    SQL_NNC_NON_NULL = 1,

//
// Null collation
//
    SQL_NC_HIGH = 0,
    SQL_NC_LOW = 1,
    SQL_NC_START = 2,
    SQL_NC_END = 4,

// File usage
//
    SQL_FILE_NOT_SUPPORTED = 0,
    SQL_FILE_TABLE = 1,
    SQL_FILE_QUALIFIER = 2,

// SQLGetData extensions masks
//
    SQL_GD_ANY_COLUMN = 0x1,
    SQL_GD_ANY_ORDER = 0x2,
    SQL_GD_BLOCK = 0x4,
    SQL_GD_BOUND = 0x8,

// Alter table
//
    SQL_AT_ADD_COLUMN = 1,
    SQL_AT_DROP_COLUMN = 2,

// Positioned statements masks
//
    SQL_PS_POSITIONED_DELETE = 0x1,
    SQL_PS_POSITIONED_UPDATE = 0x2,
    SQL_PS_SELECT_FOR_UPDATE = 0x4,

// Group By
//
    SQL_GB_NOT_SUPPORTED = 0,
    SQL_GB_GROUP_BY_EQUALS_SELECT = 1,
    SQL_GB_GROUP_BY_CONTAINS_SELECT = 2,
    SQL_GB_NO_RELATION = 3,

// Owner usage masks
//
    SQL_OU_DML_STATEMENTS = 0x1,
    SQL_OU_PROCEDURE_INVOCATION = 0x2,
    SQL_OU_TABLE_DEFINITION = 0x4,
    SQL_OU_INDEX_DEFINITION = 0x8,
    SQL_OU_PRIVILEGE_DEFINITION = 0x10,

// Qualifier usage masks
//
    SQL_QU_DML_STATEMENTS = 0x1,
    SQL_QU_PROCEDURE_INVOCATION = 0x2,
    SQL_QU_TABLE_DEFINITION = 0x4,
    SQL_QU_INDEX_DEFINITION = 0x8,
    SQL_QU_PRIVILEGE_DEFINITION = 0x10,

// Subqueries masks
//
    SQL_SQ_COMPARISON = 0x1,
    SQL_SQ_EXISTS = 0x2,
    SQL_SQ_IN = 0x4,
    SQL_SQ_QUANTIFIED = 0x8,
    SQL_SQ_CORRELATED_SUBQUERIES = 0x10,

// Union masks
//
    SQL_U_UNION = 0x1,
    SQL_U_UNION_ALL = 0x2,

// Bookmark persistence
//
    SQL_BP_CLOSE = 0x1,
    SQL_BP_DELETE = 0x2,
    SQL_BP_DROP = 0x4,
    SQL_BP_TRANSACTION = 0x8,
    SQL_BP_UPDATE = 0x10,
    SQL_BP_OTHER_HSTMT = 0x20,
    SQL_BP_SCROLL = 0x40,

// Static sensitivity
//
    SQL_SS_ADDITIONS = 0x1,
    SQL_SS_DELETIONS = 0x2,
    SQL_SS_UPDATES = 0x4,

// Lock types masks
//
    SQL_LCK_NO_CHANGE = 0x1,
    SQL_LCK_EXCLUSIVE = 0x2,
    SQL_LCK_UNLOCK = 0x4,

// Positioned operations masks
//
    SQL_POS_POSITION = 0x1,
    SQL_POS_REFRESH = 0x2,
    SQL_POS_UPDATE = 0x4,
    SQL_POS_DELETE = 0x8,
    SQL_POS_ADD = 0x10,

// Qualifier location
//
    SQL_QL_START = 1,
    SQL_QL_END = 2,



// Options for SQLGetStmtOption/SQLSetStmtOption
//
    SQL_QUERY_TIMEOUT = 0,
    SQL_MAX_ROWS = 1,
    SQL_NOSCAN = 2,
    SQL_MAX_LENGTH = 3,
    SQL_ASYNC_ENABLE = 4,
    SQL_BIND_TYPE = 5,
    SQL_CURSOR_TYPE = 6,
    SQL_CONCURRENCY = 7,
    SQL_KEYSET_SIZE = 8,
    SQL_ROWSET_SIZE = 9,
    SQL_SIMULATE_CURSOR = 10,
    SQL_RETRIEVE_DATA = 11,
    SQL_USE_BOOKMARKS = 12,
    SQL_GET_BOOKMARK = 13,
    SQL_ROW_NUMBER = 14,

    SQL_STMT_OPT_MAX = SQL_ROW_NUMBER,

// Statement option values & defaults
//
    SQL_QUERY_TIMEOUT_DEFAULT = 0,
    SQL_MAX_ROWS_DEFAULT = 0,

    SQL_NOSCAN_OFF = 0,
    SQL_NOSCAN_ON = 1,
    SQL_NOSCAN_DEFAULT = SQL_NOSCAN_OFF,

    SQL_MAX_LENGTH_DEFAULT = 0,

    SQL_ASYNC_ENABLE_OFF = 0,
    SQL_ASYNC_ENABLE_ON = 1,
    SQL_ASYNC_ENABLE_DEFAULT = SQL_ASYNC_ENABLE_OFF,

    SQL_BIND_BY_COLUMN = 0,

    SQL_CONCUR_READ_ONLY = 1,
    SQL_CONCUR_LOCK = 2,
    SQL_CONCUR_ROWVER = 3,
    SQL_CONCUR_VALUES = 4,
    SQL_CURSOR_FORWARD_ONLY = 0,
    SQL_CURSOR_KEYSET_DRIVEN = 1,
    SQL_CURSOR_DYNAMIC = 2,
    SQL_CURSOR_STATIC = 3,

    SQL_ROWSET_SIZE_DEFAULT = 1,

    SQL_KEYSET_SIZE_DEFAULT = 0,

    SQL_SC_NON_UNIQUE = 0,
    SQL_SC_TRY_UNIQUE = 1,
    SQL_SC_UNIQUE = 2,

    SQL_RD_OFF = 0,
    SQL_RD_ON = 1,
    SQL_RD_DEFAULT = SQL_RD_ON,

    SQL_UB_OFF = 0,
    SQL_UB_ON = 1,
    SQL_UB_DEFAULT = SQL_UB_ON,


// Options for SQLSetConnectOption/SQLGetConnectOption
//
    SQL_ACCESS_MODE = 101,
    SQL_AUTOCOMMIT = 102,
    SQL_LOGIN_TIMEOUT = 103,
    SQL_OPT_TRACE = 104,
    SQL_OPT_TRACEFILE = 105,
    SQL_TRANSLATE_DLL = 106,
    SQL_TRANSLATE_OPTION = 107,
    SQL_TXN_ISOLATION = 108,
    SQL_CURRENT_QUALIFIER = 109,
    SQL_CONNECT_OPT_DRVR_START = 1000,
//
    SQL_ODBC_CURSORS = 110,
    SQL_QUIET_MODE = 111,
    SQL_PACKET_SIZE = 112,

    SQL_CONN_OPT_MAX = SQL_PACKET_SIZE,

    SQL_CONN_OPT_MIN = SQL_ACCESS_MODE,

// Access mode options
//
    SQL_MODE_READ_WRITE = 0,
    SQL_MODE_READ_ONLY = 1,
    SQL_MODE_DEFAULT = SQL_MODE_READ_WRITE,

// Autocommit options
//
    SQL_AUTOCOMMIT_OFF = 0,
    SQL_AUTOCOMMIT_ON = 1,
    SQL_AUTOCOMMIT_DEFAULT = SQL_AUTOCOMMIT_ON,

// Login timeout options
//
    SQL_LOGIN_TIMEOUT_DEFAULT = 15,

// Trace options
//
    SQL_OPT_TRACE_OFF = 0,
    SQL_OPT_TRACE_ON = 1,
    SQL_OPT_TRACE_DEFAULT = SQL_OPT_TRACE_OFF,
//    SQL_OPT_TRACE_FILE_DEFAULT = "\\SQL.LOG",

//
// Cursor options
//
    SQL_CUR_USE_IF_NEEDED = 0,
    SQL_CUR_USE_ODBC = 1,
    SQL_CUR_USE_DRIVER = 2,
    SQL_CUR_DEFAULT = SQL_CUR_USE_DRIVER,


// Column types and scopes in SQLSpecialColumns.
//
    SQL_BEST_ROWID = 1,
    SQL_ROWVER = 2,

    SQL_SCOPE_CURROW = 0,
    SQL_SCOPE_TRANSACTION = 1,
    SQL_SCOPE_SESSION = 2,

// Level 2 Functions
//
// SQLExtendedFetch "fFetchType" values
//
    SQL_FETCH_NEXT = 1,
    SQL_FETCH_FIRST = 2,
    SQL_FETCH_LAST = 3,
    SQL_FETCH_PRIOR = 4,
    SQL_FETCH_ABSOLUTE = 5,
    SQL_FETCH_RELATIVE = 6,
    SQL_FETCH_BOOKMARK = 8,


// SQLExtendedFetch "rgfRowStatus" element values
//
    SQL_ROW_SUCCESS = 0,
    SQL_ROW_DELETED = 1,
    SQL_ROW_UPDATED = 2,
    SQL_ROW_NOROW = 3,
//
    SQL_ROW_ADDED = 4,
    SQL_ROW_ERROR = 5,


// Defines for SQLForeignKeys (returned in result set)
//
    SQL_CASCADE = 0,
    SQL_RESTRICT = 1,
    SQL_SET_NULL = 2,

// Defines for SQLProcedureColumns (returned in the result set)
//
    SQL_PARAM_TYPE_UNKNOWN = 0,
    SQL_PARAM_INPUT = 1,
    SQL_PARAM_INPUT_OUTPUT = 2,
    SQL_RESULT_COL = 3,
//
    SQL_PARAM_OUTPUT = 4,

// Defines for SQLStatistics
//
    SQL_INDEX_UNIQUE = 0,
    SQL_INDEX_ALL = 1,
    SQL_ENSURE = 1,
    SQL_QUICK = 0,

// Defines for SQLStatistics (returned in the result set)
//
    SQL_TABLE_STAT = 0,
    SQL_INDEX_CLUSTERED = 1,
    SQL_INDEX_HASHED = 2,
    SQL_INDEX_OTHER = 3,

//
// Procedures
//
    SQL_PT_UNKNOWN = 0,
    SQL_PT_PROCEDURE = 1,
    SQL_PT_FUNCTION = 2,

// Procedure columns
//
    SQL_PC_UNKNOWN = 0,
    SQL_PC_NON_PSEUDO = 1,
    SQL_PC_PSEUDO = 2,


// Defines for SQLSetPos
//
    SQL_ENTIRE_ROWSET = 0,

    SQL_POSITION = 0,
    SQL_REFRESH = 1,
    SQL_UPDATE = 2,
    SQL_DELETE = 3,
    SQL_ADD = 4,


// Lock options
//
    SQL_LOCK_NO_CHANGE = 0,
    SQL_LOCK_EXCLUSIVE = 1,
    SQL_LOCK_UNLOCK = 2,


// Deprecated global constants
//
    SQL_DATABASE_NAME = 16,
    SQL_FD_FETCH_PREV = SQL_FD_FETCH_PRIOR,
    SQL_FETCH_PREV = SQL_FETCH_PRIOR,
    SQL_CONCUR_TIMESTAMP = SQL_CONCUR_ROWVER,
    SQL_SCCO_OPT_TIMESTAMP = SQL_SCCO_OPT_ROWVER,
    SQL_CC_DELETE = SQL_CB_DELETE,
    SQL_CR_DELETE = SQL_CB_DELETE,
    SQL_CC_CLOSE = SQL_CB_CLOSE,
    SQL_CR_CLOSE = SQL_CB_CLOSE,
    SQL_CC_PRESERVE = SQL_CB_PRESERVE,
    SQL_CR_PRESERVE = SQL_CB_PRESERVE,
    SQL_FETCH_RESUME = 7,
    SQL_SCROLL_FORWARD_ONLY = 0,
    SQL_SCROLL_KEYSET_DRIVEN = -1,
    SQL_SCROLL_DYNAMIC = -2,
    SQL_SCROLL_STATIC = -3
}


/*
//
//  Date/Time/Timestamp Structs
//
Type DATE_STRUCT
  year      As Integer
  month     As Integer
  day       As Integer
End Type

Type TIME_STRUCT
  hour      As Integer
  minute    As Integer
  second    As Integer
End Type

Type TIMESTAMP_STRUCT
  year      As Integer
  month     As Integer
  day       As Integer
  hour      As Integer
  minute    As Integer
  second    As Integer
  fraction 
End Type
*/