﻿/*
 * Written by Stewart Berman 2012
 * Please send any bug reports / suggestions to sab@saberman.com
 * 
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Globalization;
using Microsoft.Win32;

namespace ODBC
{
    /// <summary>
    /// Interaction logic for frmManualDataSources.xaml
    /// </summary>
    public partial class winManualDataSources : Window
    {
        clsODBCReferences CODBCReferences = clsODBCReferences.Instance;

        Cursor OldCursor;

        int nDrivers;
        int nDriversLenMax;

        String sActiveDriver = "";

        Hashtable ODBCSources = new Hashtable();

        public winManualDataSources()
        {
            InitializeComponent();
            InitializeDrivers();
        }
        private void btnExit_Click(object sender, RoutedEventArgs e)
        {
            Application.Current.Shutdown();
        }
        private void btnGetInformation_Click(object sender, RoutedEventArgs e)
        {
            clsODBC CODBC;

            if ((0 >= lbDriversOpen.Items.Count) || (0 > lbDriversOpen.SelectedIndex))
            {
                MessageBox.Show("Please select an open data source and try again.", "Information About ODBC", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return;
            }

            //  Disable all of the buttons and change the cursor
            DisableAllButtonsOnForm();

            try
            {
                //  Get the data source associated with the selected entry
                sActiveDriver = lbDriversOpen.SelectedItem.ToString().Trim();
                CODBC = (clsODBC)ODBCSources[sActiveDriver];
                //  Find out which functions are supported
                CODBC.Functions();
                //  Update the function listbox
                int nFunction = 0;
                for (int nIndex = 1; nIndex <= CODBC.FunctionsUsed(); nIndex++)
                {
                    //  For each function that we setup a point to update the entry in the listbox
                    if (CODBC.FunctionExists(nIndex))
                    {
                        nFunction++;
                        if (CODBC.Function(nIndex))
                            lbFunctions.Items[nFunction] = lbFunctions.Items[nFunction].ToString().Substring(0, 31) + "Yes";
                        else
                            lbFunctions.Items[nFunction] = lbFunctions.Items[nFunction].ToString().Substring(0, 31) + "No";
                        //Debug.Print(nFunction.ToString("000") + "|" + lbFunctions.Items[nFunction].ToString().Substring(0, 31));
                    }
                    else
                    {
                        //Debug.Print("Function[" + nIndex.ToString() + "]: Does not exist.");
                    }
                }
                //  Get information values
                CODBC.Information();
                // Get information items
                int nInformation = 0;
                for (int nIndex = 1; nIndex < CODBC.InformationUsed(); nIndex++)
                {
                    if (!(CODBC.InformationName(nIndex) == null))
                    {
                        nInformation++;
                        if (0 != CODBCReferences.nInformation[nInformation])
                        {
                            lbInformation.Items[(int)CODBCReferences.nInformation[nInformation]] = lbInformation.Items[(int)CODBCReferences.nInformation[nInformation]].ToString().Substring(0, 31) + CODBC.InformationString((int)CODBCReferences.nInformation[nInformation]);
                            //Debug.Print(nInformation.ToString() + "|" + lbInformation.Items[(int)CODBCReferences.nInformation[nInformation]].ToString());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error: " + ex.Message, "Information About ODBC", MessageBoxButton.OK, MessageBoxImage.Exclamation);
            }
            finally
            {
                CODBC = null;
                //  Enable all of the buttons and restore the cursor
                EnableAllButtonsOnForm();
                lbDriversOpen.UnselectAll();
            }
        }
        private void btnOpenConnection_Click(object sender, RoutedEventArgs e)
        {
            clsODBC CODBC;

            //  Do we have a driver selected?
            if ((0 >= lbDrivers.Items.Count) || (0 > lbDrivers.SelectedIndex))
            {
                MessageBox.Show("Please select a driver and try again.", "Information for ODBC", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return;
            }
            //  Do we have a server, database and userid filled in?
            if ((0 == txtDatabase.Text.ToString().Length) || (0 == txtServer.Text.ToString().Length) || (0 == txtUserid.Text.ToString().Length))
            {
                MessageBox.Show("Server, Database and UserID are required.", "Information for ODBC", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return;
            }

            //  Disable all of the buttons and change the cursor
            DisableAllButtonsOnForm();

            try
            {
                //  Allocate an env and connection
                CODBC = new clsODBC(CODBCReferences);
                String sDriver = lbDrivers.SelectedItem.ToString().Substring(0, nDriversLenMax).Trim();
                CODBC.AllocEnv();
                CODBC.AllocConnect();
                //  Connect to the database
                if (CODBC.DriverConnect(sDriver, txtServer.Text.ToString(), txtDatabase.Text.ToString(), txtUserid.Text.ToString(), txtPassword.Password.ToString()))
                {
                    //  Connection successfull -- allocate a statement and add the data source to the collection
                    lbDriversOpen.Items.Add(sDriver + "|" + txtServer.Text.ToString() + "|" + txtDatabase.Text.ToString());
                    CODBC.AllocStmt();
                    ODBCSources.Add(sDriver + "|" + txtServer.Text.ToString() + "|" + txtDatabase.Text.ToString(), CODBC);
                }
            }
            catch (Exception ex)
            {
                //  Caught an error -- destroy the data source object, tell the user and get out
                MessageBox.Show("Error: " + ex.Message, "Information About ODBC", MessageBoxButton.OK, MessageBoxImage.Exclamation);
            }
            finally
            {
                CODBC = null;
                lbDrivers.UnselectAll();
                //  Enable all of the buttons and restore the cursor
                EnableAllButtonsOnForm();
            }
        }
        private void btnSaveInformation_Click(object sender, RoutedEventArgs e)
        {
            if (0 == sActiveDriver.Length)
            {
                MessageBox.Show("Please retrieve information before trying to save it.", "Information for ODBC", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return;
            }
            // Configure open file dialog box
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.CheckPathExists = true;
            ofd.CheckFileExists = false;
            ofd.DefaultExt = ".txt";
            ofd.FileName = sActiveDriver.Replace("|","_") + "_" + DateTime.Now.ToString("yyyyMMddHHmmss");
            ofd.Filter = "Text documents (.txt)|*.txt|All files (*.*)|*.*";
            ofd.FilterIndex = 1;
            ofd.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
            ofd.Multiselect = false;
            ofd.Title = "Information for ODBC";

            // Show open file dialog box
            Nullable<bool> result = ofd.ShowDialog();

            // Process open file dialog box results
            if (result == true)
            {
                try
                {
                    // Get data source
                    clsODBC CODBC = (clsODBC)ODBCSources[sActiveDriver];
                    // Open document
                    StreamWriter sw = new StreamWriter(ofd.FileName, false);
                    sw.WriteLine("Information About ODBC");
                    sw.WriteLine("Data Soure: " + sActiveDriver);
                    sw.WriteLine("Connect String: " + CODBC.ConnectStringOut());
                    sw.WriteLine();
                    for (int nIndex = 0; nIndex<lbFunctions.Items.Count; nIndex++)
                        sw.WriteLine(lbFunctions.Items[nIndex].ToString());
                    sw.WriteLine();
                    for (int nIndex = 0; nIndex < lbInformation.Items.Count; nIndex++)
                        sw.WriteLine(lbInformation.Items[nIndex].ToString());
                    sw.Close();
                    sw = null;
                    CODBC = null;
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error: " + ex.Message, "Information About ODBC", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                }
            }
        }
        private void DisableAllButtonsOnForm()
        {
            //  Disable the buttons while we do something so the user doesn't accidently click one
            this.btnExit.IsEnabled = false;
            this.btnGetInformation.IsEnabled = false;
            this.btnOpenConnection.IsEnabled = false;
            this.btnSaveInformation.IsEnabled = false;
            // Save the old mouse pointer and change it to busy
            OldCursor = Mouse.OverrideCursor;
            Mouse.OverrideCursor = Cursors.Wait;
        }
        private void EnableAllButtonsOnForm()
        {
            //  Enable the buttons after we finish what we were doing
            this.btnExit.IsEnabled = true;
            this.btnGetInformation.IsEnabled = true;
            this.btnOpenConnection.IsEnabled = true;
            this.btnSaveInformation.IsEnabled = true;
            //  Reset the cursor
            Mouse.OverrideCursor = OldCursor;
        }
        private void InitializeDrivers()
        {
            clsODBC CODBC;

            int nIndex;

            string[] sDrivers;
            string[] sDescriptons;

            // Create the clsODBC class and have it get the drivers
            CODBC = new clsODBC(CODBCReferences);
            nDrivers = CODBC.Drivers();
            nDriversLenMax = 0;
            //  Do we have any drivers
            if (0 < nDrivers)
            {
                //  Yes -- copy the names and drivers into our arrays
                sDrivers = new string[nDrivers];
                sDescriptons = new string[nDrivers];
                for (nIndex = 0; nIndex < nDrivers; nIndex++)
                {
                    sDrivers[nIndex] = CODBC.Driver(nIndex);
                    sDescriptons[nIndex] = CODBC.Attributes(nIndex);
                    //  We need the maximum length of the driver names for formatting
                    if (nDriversLenMax < sDrivers[nIndex].Length)
                        nDriversLenMax = sDrivers[nIndex].Length;
                }
                //  Need to padd out the driver names
                String sPadding = new String(' ', nDriversLenMax);
                //  Combine the driver names and their attributes and sort them
                SortedSet<String> sl = new SortedSet<String>();
                for (nIndex = 0; nIndex < nDrivers; nIndex++)
                    sl.Add(sDrivers[nIndex] + sPadding.Substring(0, 1 + nDriversLenMax - sDrivers[nIndex].Length) + sDescriptons[nIndex]);
                //  Put headings in listbox
                lbDrivers.Items.Add("Data Source Name" + sPadding.Substring(0, 1 + nDriversLenMax - "Data Source Name".Length) + "Description");
                //  Put drivers and their attributes in listbox
                for (nIndex = 0; nIndex < nDrivers; nIndex++)
                    lbDrivers.Items.Add(sl.ElementAt(nIndex).ToString());

                //  Need to padd out the function and information names
                sPadding = new String(' ', 32);
                //  Populate the functions listbox
                //  Put headings in listbox
                lbFunctions.Items.Add("Function Name" + sPadding.Substring(0, 31 - "Function Name".Length) + "Supported");
                //  Put functions in listbox
                int nFunctionsUsed = CODBC.FunctionsUsed();
                for (nIndex = 1; nIndex <= nFunctionsUsed; nIndex++)
                {
                    if (!(CODBC.FunctionName(nIndex) == null))
                        lbFunctions.Items.Add(CODBC.FunctionName(nIndex) + sPadding.Substring(0, 31 - CODBC.FunctionName(nIndex).Length));
                }

                //  Populate the information listbox
                //  Put headings in listbox
                lbInformation.Items.Add("Information Type" + sPadding.Substring(0, 31 - "Information Type".Length) + "Information");
                //  Put information names in listbox
                int nInformationUsed = CODBC.InformationUsed();
                for (nIndex = 1; nIndex <= nInformationUsed; nIndex++)
                    if (!(CODBC.InformationName(nIndex) == null))
                        lbInformation.Items.Add(CODBC.InformationName(nIndex) + sPadding.Substring(31 - CODBC.InformationName(nIndex).ToString().Length));
            }
        }
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            string sDriver;
            for (int nIndex = 1; nIndex < lbDrivers.Items.Count; nIndex++)
            {
                sDriver = (string)lbDrivers.Items[nIndex];
                ODBCSources.Remove(sDriver.Substring(0, nDriversLenMax).Trim());
            }
            CODBCReferences = null;
            Application.Current.Shutdown();
        }
        private void lbDriversOpen_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            clsODBC CODBC;

            if ((0 >= lbDriversOpen.Items.Count) || (0 > lbDriversOpen.SelectedIndex))
            {
                MessageBox.Show("Please select an open data source and try again.", "Information About ODBC", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return;
            }

            //  Disable all of the buttons and change the cursor
            DisableAllButtonsOnForm();

            try
            {
                //  Get the data source associated with the selected entry
                CODBC = (clsODBC)ODBCSources[lbDriversOpen.SelectedItem.ToString().Trim()];
                MessageBox.Show(CODBC.ConnectStringOut(), "Information About ODBC", MessageBoxButton.OK, MessageBoxImage.Information);
            }
            catch (Exception ex)
            {
                //  Caught an error -- destroy the data source object, tell the user and get out
                MessageBox.Show("Error: " + ex.Message, "Information About ODBC", MessageBoxButton.OK, MessageBoxImage.Exclamation);
            }
            finally
            {
                CODBC = null;
                lbDrivers.UnselectAll();
                //  Enable all of the buttons and restore the cursor
                EnableAllButtonsOnForm();
            }
        }
    }
}
